Pokedex = LibStub("AceAddon-3.0"):NewAddon("Pokedex", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Pokedex")

--=========================================================================--
-- Keybinding globals --
--=========================================================================--
BINDING_HEADER_POKEDEX = L["Pokedex"]
BINDING_NAME_POKEDEXSUPERTOGGLE = L["Super Toggle"]
BINDING_NAME_POKEDEXSUMMONMOUNT = L["Summon Mount"]
BINDING_NAME_POKEDEXDISMISSMOUNT = L["Dismiss Mount"]
BINDING_NAME_POKEDEXTOGGLEMOUNT = L["Toggle Mount"]
BINDING_NAME_POKEDEXSUMMONNEXTMOUNT = L["Summon Next Mount"]
BINDING_NAME_POKEDEXSUMMONOTHERMOUNT = L["Summon Other Mount"]
BINDING_NAME_POKEDEXSUMMONCOMPANION = L["Summon Companion"]
BINDING_NAME_POKEDEXDISMISSCOMPANION = L["Dismiss Companion"]
BINDING_NAME_POKEDEXTOGGLECOMPANION = L["Toggle Companion"]
BINDING_NAME_POKEDEXSUMMONNEXTCOMPANION = L["Summon Next Companion"]

--=========================================================================--
-- Fake Enum Types		TODO: find a way to make these constants
--=========================================================================--
-- Title Info -- 
local TI_NAME = 1
local TI_ID   = 2
local TI_RANK = 3

-- Companion Info -- 
local CI_NAME = 1
local CI_ID   = 2
local CI_RANK = 3
local CI_SNOWBALL = 4

-- Mount Info -- 
local MI_NAME  = 1
local MI_ID    = 2
local MI_RANK  = 3

-- Mount Speeds
local MS_Unknown      =  0  -- mounts we failed to identify
local MS_ATV          =  1	-- mounts that can fly and run in old world based on rider skill like the Headless Horseman's Mount
local MS_Flyer310     =  2	-- extremely fast flyers like the Ashes of Al'ar
local MS_Flyer280     =  3	-- very fast flyers
local MS_FlyerSkill   =  4	-- flyers where speed is based on rider skill
local MS_Flyer60      =  5	-- regular flyers
local MS_Runner100    =  6	-- very fast running mounts
local MS_RunnerSkill  =  7	-- runners where speed is based on rider skill like the Big Blizzard Bear
local MS_Runner60     =  8	-- regular runners
local MS_Runner0      =  9	-- running mounts that don't actually increase speed like the Riding Turtle
local MS_RunnerQiraji =  10	-- Qiraji scarab mounts

-- Mounts by type info
local MOUNT_INDICES = 1
local MOUNT_NAMES = 2

--=========================================================================--
-- Announcement Info 
--=========================================================================--
local g_rgChannelDescs = { L["personal"], L["party"], L["raid"], L["emote"], L["say"], L["yell"] }
local g_rgChannels = { "ERROR", "PARTY", "RAID", "EMOTE", "SAY", "YELL" } -- CHATID types, not localized, ERROR is just placeholder

--=========================================================================--
--=========================================================================--

local g_iRidingSkill = 0
local g_iCurrentMount = 0
local g_rgstrNoMounts = { [0] = L["no mounts available"] }
local g_rgstrMountTypes		-- indexed by mount speed, not contiguously from 0 or 1
local g_iMountType			-- current index showing in dropdown, will be 0 or MS_ value
local g_MountsByType		-- g_MountsByType[MS_speed][MOUNT_INDICES|MOUNT_NAMES][array]
local g_iMountOfType		-- standard index
local g_rgstrHotMountNames
local g_iHotMount

local g_iCurrentCompanion = 0
local g_rgstrNoCompanions = { [0] = L["no companions available"] }
local g_rgstrCompanionNames
local g_iCompanionForRanking
local g_rgstrHotCompanionNames
local g_iHotCompanion

local g_rgstrNoTitles = { [0] = L["no titles available"] }
local g_rgstrTitleNames
local g_iTitleForRanking
local g_rgstrHotTitleNames
local g_iHotTitle

local g_iDefaultRank = 5
local g_iCurDataFormat = 2
local g_fInitialized = false

local g_itemIDSnowball = 17202

local defaults = {
	profile = {
		iDataFormat = 0,
		fFirstBoot = true,
		fAnnounce = true,
		iChannel = 1,
		fChangeTitleOnLoad = false,
		fEnableHotness = true,
		fNewHotness = true,
		idHotMount = 0,
		iHotMountPref = 50,
		idHotCompanion = 0,
		iHotCompanionPref = 50,
		idHotTitle = 0,
		iHotTitlePref = 50,
		rgMountInfo = {},			-- { MI_NAME, MI_ID, MI_RANK }
		rgCompanionInfo = {},		-- { CI_NAME, CI_ID, CI_RANK }
		rgTitleInfo = {}			-- { TI_NAME, TI_ID, TI_RANK }

	},
}

-- As the addon develops and ages, there may be changes to the structure 
-- and contents of the saved data. This function migrates data between
-- data format versions to preserve the users settings.
function Pokedex:MigrateSavedData()
	-- if its first boot, theres no data to migrate
	if not self.db.profile.fFirstBoot then
		
		if self.db.profile.iDataFormat < 1 then
			-- rename of critter to companion
			if self.db.profile.idHotCritter ~= nil then
				self.db.profile.idHotCompanion = self.db.profile.idHotCritter
				self.db.profile.idHotCritter = nil
			end
			if self.db.profile.idHotCritterPref ~= nil then
				self.db.profile.iHotCompanionPref = self.db.profile.idHotCritterPref
				self.db.profile.idHotCritterPref = nil
			end
			if self.db.profile.rgCritterInfo ~= nil then
				self.db.profile.rgCompanionInfo = self.db.profile.rgCritterInfo
				self.db.profile.rgCritterInfo = nil
			end
		end

		if self.db.profile.iDataFormat < 2 then
			-- change of how we characterize mount performance
			for index = 1, #self.db.profile.rgMountInfo, 1 do
				self.db.profile.rgMountInfo[index][4] = nil
			end
		end

	end

	self.db.profile.iDataFormat = g_iCurDataFormat
end


local options = {
    name = "Pokedex",
    handler = Pokedex,
    type = 'group',
    childGroups = "tab",
    args = {
		zone = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["echoes out zone info"],
			desc = L["echoes out zone info"],
			func = "EchoZone",
		},
		SuperToggle = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["toggle mount or companion"],
			desc = L["toggles mount or companion"],
			func = "SuperToggle",
		},
		SummonMount = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon mount"],
			desc = L["summons a mount"],
			func = "SummonMount",
		},
		sm = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon mount"],
			desc = L["summons a mount"],
			func = "SummonMount",
		},
		DismissMount = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["dismiss mount"],
			desc = L["dismisses current mount"],
			func = "DismissMount",
		},
		dm = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["dismiss mount"],
			desc = L["dismisses current mount"],
			func = "DismissMount",
		},
		ToggleMount = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["toggle mount"],
			desc = L["toggles a mount"],
			func = "ToggleMount",
		},
		tm = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["toggle mount"],
			desc = L["toggles a mount"],
			func = "ToggleMount",
		},
		SummonNextMount = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon next mount"],
			desc = L["summons next mount in collection"],
			func = "SummonNextMount",
		},
		snm = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon next mount"],
			desc = L["summons next mount in collection"],
			func = "SummonNextMount",
		},
		SummonOtherMount = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon other mount"],
			desc = L["if summon mount would summon flyer, will summon ground mount and vice versa"],
			func = "SummonOtherMount",
		},
		som = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon other mount"],
			desc = L["if summon mount would summon flyer, will summon ground mount and vice versa"],
			func = "SummonOtherMount",
		},
		SummonCompanion = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon companion"],
			desc = L["summons a companion"],
			func = "SummonCompanion",
		},
		sc = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon companion"],
			desc = L["summons a companion"],
			func = "SummonCompanion",
		},
		DismissCompanion = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["dismiss companion"],
			desc = L["dismisses current companion"],
			func = "DismissCompanion",
		},
		dc = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["dismiss companion"],
			desc = L["dismisses current companion"],
			func = "DismissCompanion",
		},
		ToggleCompanion = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["toggle companion"],
			desc = L["toggles a companion"],
			func = "ToggleCompanion",
		},
		tc = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["toggle companion"],
			desc = L["toggles a companion"],
			func = "ToggleCompanion",
		},
		SummonNextCompanion = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon next companion"],
			desc = L["summons next companion in collection"],
			func = "SummonNextCompanion",
		},
		snc = {
			hidden = true,
			order = 0,
			type = "execute",
			name = L["summon next companion"],
			desc = L["summons next companion in collection"],
			func = "SummonNextCompanion",
		},

		CtrlReadMe = {
			order = 6,
			type = "description",
			name = L["Please see the README.TXT file in the Pokedex addon folder for more information on how to use Pokedex"],
		},
		
		CtrlAnnounce = {
			order = 11,
			type = "toggle",
			name = L["announce"],
			desc = L["(on|off) let everyone know who *you* choose"],
			get = "FGetAnnounce",
			set = "SetAnnounce",
		},
		CtrlChannel = {
			order = 12,
			type = "select",
			name = L["channel"],
			desc = L["channel to announce selection in"],
			values = g_rgChannelDescs,
			get = "GetChannel",
			set = "SetChannel",
		},
		CtrlEnableHotness = {
			order = 13,
			type = "toggle",
			name = L["enable hot pets"],
			desc = L["lets your turn on|off the hot pet subfeatures"],
			get = "FGetEnableHotness",
			set = "SetEnableHotness",
		},
		CtrlNewHotness = {
			order = 14,
			type = "toggle",
			name = L["new hotness"],
			desc = L["always make newest mount or companion the hot one"],
			disabled = "FGetDisableHotness",
			get = "FGetNewHotness",
			set = "SetNewHotness",
		},

		CtrlGroupMounts = {
			order = 20,
			name = "Mounts",
			handler = Pokedex,
			type = 'group',
			args = {
				CtrlSummonMount = {
					order = 1,
					type = "execute",
					name = L["summon mount"],
					desc = L["summons a mount"],
					func = "SummonMount",
				},
				CtrlDismissMount = {
					order = 2,
					type = "execute",
					name = L["dismiss mount"],
					desc = L["dismisses current mount"],
					func = "DismissMount",
				},
				CtrlMountTypes = {
					order = 5,
					type = "select",
					name = L["select mount type"],
					desc = L["select mount type"],
					values = "MountTypeValues",
					get = "GetMountType",
					set = "SetMountType",
				},
				CtrlPaddingMountTypes = {
					order = 6,
					type = "description",
					name = " ",
				},
				CtrlMountsOfType = {
					order = 7,
					type = "select",
					name = L["select mount for ranking"],
					desc = L["mount whose rank you can set"],
					values = "MountsOfTypeValues",
					get = "GetMountOfType",
					set = "SetMountOfType",
				},
				CtrlRankingForMount = {
					order = 8,
					type = "range",
					name = L["mount rank"],
					desc = L["rank of current mount"],
					min = 0,
					max = 10,
					step = 1,
					disabled = "FDisablePerMountSettings",
					get = "GetRankingForMount",
					set = "SetRankingForMount",
				},
				CtrlHotMount = {
					order = 11,
					type = "select",
					name = L["select hot mount"],
					desc = L["mount to become hot one"],
					values = "HotMountValues",
					disabled = "FGetDisableHotness",
					get = "GetHotMount",
					set = "SetHotMount",
				},
				CtrlMountHeat = {
					order = 12,
					type = "range",
					name = L["mount heat"],
					desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
					min = 0,
					max = 100,
					step = 5,
					disabled = "FGetDisableHotness",
					get = "GetMountHeat",
					set = "SetMountHeat",
				},
			},
		},
		CtrlGroupCompanions = {
			order = 40,
			name = "Companions",
			handler = Pokedex,
			type = 'group',
			args = {
				CtrlSummonCompanion = {
					order = 2,
					type = "execute",
					name = L["summon companion"],
					desc = L["summons a companion"],
					func = "SummonCompanion",
				},
				CtrlDismissCompanion = {
					order = 3,
					type = "execute",
					name = L["dismiss companion"],
					desc = L["dismisses current companion"],
					func = "DismissCompanion",
				},
				CtrlCompanionForRanking = {
					order = 4,
					type = "select",
					name = L["select companion for ranking"],
					desc = L["companion whose rank you can set"],
					values = "CompanionForRankingValues",
					get = "GetCompanionForRanking",
					set = "SetCompanionForRanking",
				},
				CtrlRankingForCompanion = {
					order = 5,
					type = "range",
					name = L["companion rank"],
					desc = L["rank of current companion"],
					min = 0,
					max = 10,
					step = 1,
					disabled = "FDisableRankingForCompanion",
					get = "GetRankingForCompanion",
					set = "SetRankingForCompanion",
				},
				CtrlHotCompanion = {
					order = 6,
					type = "select",
					name = L["select hot companion"],
					desc = L["companion to become hot one"],
					values = "HotCompanionValues",
					disabled = "FGetDisableHotness",
					get = "GetHotCompanion",
					set = "SetHotCompanion",
				},
				CtrlCompanionHeat = {
					order = 7,
					type = "range",
					name = L["companion heat"],
					desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
					min = 0,
					max = 100,
					step = 5,
					disabled = "FGetDisableHotness",
					get = "GetCompanionHeat",
					set = "SetCompanionHeat",
				},
			},
		},
		CtrlGroupTitles = {
			order = 60,
			name = "Titles",
			handler = Pokedex,
			type = 'group',
			args = {
				CtrlChangeTitleOnLoad = {
					order = 2,
					type = "toggle",
					name = L["Change title on load"],
					desc = L["Change title when logging in or reloading UI"],
					get = "FGetChangeTitleOnLoad",
					set =  "SetChangeTitleOnLoad",
				},
				CtrlChangeTitle = {
					order = 3,
					type = "execute",
					name = L["change title"],
					desc = L["change title"],
					func = "ChangeTitle",
				},
				CtrlTitleForRanking = {
					order = 4,
					type = "select",
					name = L["select title for ranking"],
					desc = L["title whose rank you can set"],
					values = "TitleForRankingValues",
					get = "GetTitleForRanking",
					set = "SetTitleForRanking",
				},
				CtrlRankingForTitle = {
					order = 5,
					type = "range",
					name = L["title rank"],
					desc = L["rank of current title"],
					min = 0,
					max = 10,
					step = 1,
					disabled = "FDisableRankingForTitle",
					get = "GetRankingForTitle",
					set = "SetRankingForTitle",
				},
				CtrlHotTitle = {
					order = 6,
					type = "select",
					name = L["select hot title"],
					desc = L["title to become hot one"],
					values = "HotTitleValues",
					disabled = "FGetDisableHotness",
					get = "GetHotTitle",
					set = "SetHotTitle",
				},
				CtrlTitleHeat = {
					order = 7,
					type = "range",
					name = L["title heat"],
					desc = L["set hotness as a percentage - 100% means the hot pet is the only one that will be chosen"],
					min = 0,
					max = 100,
					step = 5,
					disabled = "FGetDisableHotness",
					get = "GetTitleHeat",
					set = "SetTitleHeat",
				},
			},
		},
    },
}


-- Called when the addon is loaded
function Pokedex:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("PokedexDB", defaults)

	--self.optionsFrame = LibStub("AceConfig-3.0"):AddToBlizOptions("Pokedex", "Pokedex")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Pokedex", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Pokedex", "Pokedex")
	self:RegisterChatCommand("pd", "ChatCommand")
	self:RegisterChatCommand("pokedex", "ChatCommand")
end


function Pokedex:ChatCommand(input)
	if not input or input:trim() == "" then
		InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
		--LibStub("AceConfigDialog-3.0"):Open("Pokedex")
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(Pokedex, "pd", "Pokedex", input)
    end
end


-- Called when the addon is enabled
function Pokedex:OnEnable()
	CreateFrame( "GameTooltip", "PokedexScanningTooltip" ); -- Tooltip name cannot be nil
	PokedexScanningTooltip:SetOwner( WorldFrame, "ANCHOR_NONE" );
	-- Allow tooltip SetX() methods to dynamically add new lines based on these
	PokedexScanningTooltip:AddFontStrings(
		PokedexScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		PokedexScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" ) );

	-- update data when new mount or companion added	
	self:RegisterEvent("COMPANION_LEARNED")

	-- keep track of currently visible mount or companion
	self:RegisterEvent("COMPANION_UPDATE")

	-- update data when new title added	
	self:RegisterEvent("NEW_TITLE_EARNED")

	-- update mount data if riding skill changes
	self:RegisterEvent("SKILL_LINES_CHANGED")

	self:Initialize()
end

-- Initialize all the pet data in the addon
function Pokedex:Initialize()
	if (g_fInitialized) then
		self:Print("ERROR: trying to reinitialize")
		return
	end

	-- if we can't get CRITTER or MOUNT names, then we know we'll need to reload
	if (GetNumCompanions("MOUNT") > 0) then
		local creatureID, creatureName = GetCompanionInfo("MOUNT", 1)
		g_fInitialized = (creatureName ~= nil)
	elseif (GetNumCompanions("CRITTER") > 0) then
		local creatureID, creatureName = GetCompanionInfo("CRITTER", 1)
		g_fInitialized = (creatureName ~= nil)
	else
		g_fInitialized = true
	end

	-- don't bother to init because we're broken
	if (not g_fInitialized) then return end

	g_iRidingSkill = self:GetRiderSkill()

	self:MigrateSavedData()

	self:UpdateMountInfo()
	self:UpdateCompanionInfo()
	self:UpdateTitleInfo()

	self.db.profile.fFirstBoot = false
	self:EchoPetCount()
	
	-- change title on load
	if (self.db.profile.fChangeTitleOnLoad) then self:ChangeTitle() end
end

-- Called when the addon is disabled
function Pokedex:OnDisable()
end


function Pokedex:COMPANION_LEARNED()
	self:UpdateMountInfo()
	self:UpdateCompanionInfo()
end


function Pokedex:COMPANION_UPDATE()
	-- we might be in the bad cache situation
	if (not g_fInitialized) then self:Initialize() end
	
	self:GetCurrentMount()
	self:GetCurrentCompanion()
end


function Pokedex:NEW_TITLE_EARNED()
	self:UpdateTitleInfo()
end


function Pokedex:SKILL_LINES_CHANGED()
	local iNewRidingSkill = self:GetRiderSkill()
	if (iNewRidingSkill > g_iRidingSkill) then
		g_iRidingSkill = iNewRidingSkill
		self:UpdateMountInfo()
	end
end


function Pokedex:SuperToggle() 
	if IsOutdoors() then
		self:ToggleMount()
	else
		self:ToggleCompanion()
	end
end

function Pokedex:EchoPetCount()
	self:Print("Current pet counts are " .. #self.db.profile.rgMountInfo .. " mounts and " .. #self.db.profile.rgCompanionInfo .. " companions")
end

function Pokedex:EchoZone()
	SetMapToCurrentZone()
	if (IsFlyableArea()) then
		self:Print(GetCurrentMapContinent() .. "  " .. GetZoneText() .. "  " .. GetSubZoneText() .. "  Flyable")
	else
		self:Print(GetCurrentMapContinent() .. "  " .. GetZoneText() .. "  " .. GetSubZoneText() .. "  not Flyable")
	end
end

--=========================================================================--
-- Announcement functions
--=========================================================================--

function Pokedex:FGetAnnounce(info) return self.db.profile.fAnnounce; end
function Pokedex:SetAnnounce(info, fNew) self.db.profile.fAnnounce = fNew; end

function Pokedex:GetChannel(info) return self.db.profile.iChannel; end
function Pokedex:SetChannel(info, iNew)
	if (iNew >= 1 and iNew <= 6) then
		self.db.profile.iChannel = iNew
	else
		self:Print(L["ERROR: bad channel set"])
	end
end

function Pokedex:AnnounceSummon(iNew)
	if (not self.db.profile.fAnnounce) then return end

	local iChannel = self.db.profile.iChannel
	
	-- if not in raid, try sending it to party
	if (3 == iChannel and 0 == GetNumRaidMembers()) then
		iChannel = 2
	end
	-- if not in a party, just send it to yourself
	if (2 == iChannel and 0 == GetNumPartyMembers()) then
		iChannel = 1
	end
	
	if (iChannel == 1) then
		self:Print(L['X, I choose you!'](g_rgstrCompanionNames[iNew]));
	elseif (iChannel == 4) then
		SendChatMessage(L['lets X know that they have been chosen.'](g_rgstrCompanionNames[iNew]), "EMOTE");
	else
		SendChatMessage(L['X, I choose you!'](g_rgstrCompanionNames[iNew]), g_rgChannels[iChannel]);
	end
	
end

--=========================================================================--
-- general Hotness functions
--=========================================================================--

function Pokedex:FGetEnableHotness(info) return self.db.profile.fEnableHotness; end
function Pokedex:SetEnableHotness(info, fNew) self.db.profile.fEnableHotness = fNew; end
function Pokedex:FGetDisableHotness(info) return not self:FGetEnableHotness(info); end
function Pokedex:FGetNewHotness(info) return self.db.profile.fNewHotness; end
function Pokedex:SetNewHotness(info, fNew) self.db.profile.fNewHotness = fNew; end

--=========================================================================--
-- Mount functions
--=========================================================================--

function Pokedex:GetRiderSkill()
	local iMaxSkills = GetNumSkillLines()
	for iSkill = 1, iMaxSkills, 1 do
		local skillName, isHeader, isExpanded, skillRank = GetSkillLineInfo(iSkill)
		if strfind(skillName, L["Riding"]) then
			-- self:Print("Rider skill is " .. skillRank)
			return skillRank
		end
	end
	
	return 0
end

function Pokedex:SummonMount() self:SummonMountInner(false, false); end
function Pokedex:SummonNextMount() self:SummonMountInner(true, false); end
function Pokedex:SummonOtherMount() self:SummonMountInner(false, true); end

function Pokedex:SummonMountInner(fNext, fOther)
	local iCurrent = self:GetCurrentMount()
	local iMountType
	local fFlyable = false
	local fScarab = false
	
	if (iCurrent ~= 0) then
		local start, duration, enable = GetCompanionCooldown("MOUNT", iCurrent)
		if (duration ~= 0) then
			-- self:Print("in cooldown, ignore this summons attempt")
			return
		end
	end

	-- check if we can fly - cover 99% of situations
	fFlyable = IsFlyableArea() and not IsSwimming()
	
	-- check locations for special rules about what kinds of mounts we can actually summon
	SetMapToCurrentZone()
	continent = GetCurrentMapContinent()
	
	if 1 == continent then fScarab = strfind(L["Ahn'Qiraj"], GetZoneText()) end
	if 4 == continent and fFlyable then
		-- if they haven't bought cold weather flying, then they can't fly in northrend
		if not IsUsableSpell(L["Cold Weather Flying"]) then
			fFlyable = false
		-- if Wintergrasp is being contested, you can't fly there
		else
			if strfind(GetZoneText(), L["Wintergrasp"]) then
				fFlyable = (nil ~= GetWintergraspWaitTime())
			end			
		end
	end

	-- flip the type if the user has asked for that
	if fOther then fFlyable = not fFlyable end

	if fFlyable then 
		if (g_MountsByType[MS_Flyer310] ~= nil) then 
			iMountType = MS_Flyer310
		elseif (g_MountsByType[MS_Flyer280] ~= nil) then 
			iMountType = MS_Flyer280
		elseif (g_MountsByType[MS_Flyer60] ~= nil) then 
			iMountType = MS_Flyer60
		else
			fFlyable = false
		end
	end

	if not fFlyable then 
		if (fScarab and g_MountsByType[MS_RunnerQiraji] ~= nil) then 
			iMountType = MS_RunnerQiraji
		elseif (g_MountsByType[MS_Runner100] ~= nil) then 
			iMountType = MS_Runner100
		elseif (g_MountsByType[MS_Runner60] ~= nil) then 
			iMountType = MS_Runner60
		elseif (g_MountsByType[MS_Runner0] ~= nil) then 
			iMountType = MS_Runner0
		else
			self:Print(L["ERROR: You have no summonable mounts."])
			return
		end
	end
	

	local cRanks = 0
	local fCurrentMountInSet = false
	local fHotMountInSet = false
	
	-- if there's only one mount of the selected type then we know what to summon
	if (1 == #g_MountsByType[iMountType][MOUNT_INDICES]) then
		CallCompanion("MOUNT", g_MountsByType[iMountType][MOUNT_INDICES][1])
		-- TODO self:AnnounceSummon(g_MountsByType[iMountType][MOUNT_INDICES][1])
		return
	end
		
	-- check to see if the set we're summoning from contains the current mount or the hot mount
	for iMountOfType = 1, #g_MountsByType[iMountType][MOUNT_INDICES], 1 do
		local iMount = g_MountsByType[iMountType][MOUNT_INDICES][iMountOfType]
		cRanks = cRanks + self.db.profile.rgMountInfo[iMount][MI_RANK]
		
		if (iMount == g_iHotMount) then fHotMountInSet = true end
		if (iMount == iCurrent) then 
			fCurrentMountInSet = true 

			for iMountOfType2 = iMountOfType + 1, #g_MountsByType[iMountType][MOUNT_INDICES], 1 do
				local iMount2 = g_MountsByType[iMountType][MOUNT_INDICES][iMountOfType2]
				if (0 ~= self.db.profile.rgMountInfo[iMount2][MI_RANK]) then
					CallCompanion("MOUNT", iMount2)
					-- TODO self:AnnounceSummon(iMount2)
					return
				end
			end

			for iMountOfType2 = 1, iMountOfType, 1 do
				local iMount2 = g_MountsByType[iMountType][MOUNT_INDICES][iMountOfType2]
				if (0 ~= self.db.profile.rgMountInfo[iMount2][MI_RANK]) then
					CallCompanion("MOUNT", iMount2)
					-- TODO self:AnnounceSummon(iMount2)
					return
				end
			end
			
			-- only mount in set is summoned? if so we shouldn't be here
			self:Print(L["ERROR: selection error"])
			return
		end

	end

	-- hot pet check, only matters if hot pet is in set we're summoning from
	if (self.db.profile.fEnableHotness and g_iHotMount ~= 0 and fHotMountInSet and g_iHotMount ~= iCurrent) then
		-- do hot pet check and possibly summon
		if (self.db.profile.iHotMountPref >= math.random(1,100)) then
			CallCompanion("Mount", g_iHotMount)
			-- TODO self:AnnounceSummon(g_iHotMount)
			return
		end
		
		-- if you're here, they have a hot pet and its in this set, but its not coming out
		cRanks = cRanks - self.db.profile.rgMountInfo[g_iHotMount][MI_RANK]
	end
	
	-- if the current mount is in the set we're summoning out of, remove it from the ranks total
	if (iCurrent ~= 0 and fCurrentMountInSet) then
		cRanks = cRanks - self.db.profile.rgMountInfo[iCurrent][MI_RANK]
	end

	if (cRanks <= 0) then 
		if (iCurrent ~= 0) then
			return -- only summonable pet is already summoned
		end
		
		self:Print(L["ERROR: You have no summonable mounts."])
		return
	end
	
	cRanks = math.random(1,cRanks)

	for iMountOfType = 1, #g_MountsByType[iMountType][MOUNT_INDICES], 1 do
		local iNew = g_MountsByType[iMountType][MOUNT_INDICES][iMountOfType]
		if (iNew ~= iCurrent and (iNew ~= g_iHotMount or not self.db.profile.fEnableHotness)) then
			cRanks = cRanks - self.db.profile.rgMountInfo[iNew][MI_RANK]
			if (cRanks <= 0) then 
				-- found our slot
				CallCompanion("MOUNT", iNew)
				-- TODO self:AnnounceSummon(iNew)
				return
			end  
		end
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]) end
end


function Pokedex:DismissMount()
	if UnitInVehicle("player") then
		VehicleExit()
	else
		Dismount()
	end
end


function Pokedex:ToggleMount()
	if (self:GetCurrentMount() > 0 or UnitInVehicle("player")) then
		self:DismissMount()
	else
		self:SummonMount()
	end
end


function Pokedex:GetCurrentMount()
	local creatureID, creatureName, spellID, icon, active

	if (g_iCurrentMount ~= nil and g_iCurrentMount~=0) then
		creatureID, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", g_iCurrentMount)
		if (active) then 
			return g_iCurrentMount
		end 
	end
	
	g_iCurrentMount = 0 -- might not be any Mount active anymore
	
	for index = 1, GetNumCompanions("MOUNT"), 1 do
		creatureID, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", index)
		if active then 
			g_iCurrentMount = index
			break
		end
	end
	
	return g_iCurrentMount
end


function Pokedex:UpdateMountInfo()
	g_iCurrentMount = 0
	g_rgstrMountTypes = {}
	g_iMountType = 0
	g_MountsByType = {}
	g_iMountOfType = 0
	g_rgstrHotMountNames = {}
	g_iHotMount = 0
	

	-- check to see if any mounts were added when we weren't looking
	if (#self.db.profile.rgMountInfo ~= GetNumCompanions("MOUNT")) then
		-- assumption is that order doesn't change except for insertions of new pets
		-- so we can speed up searching saved data for rankings by starting the search 
		-- at a predicted index
		local iMatch = 1
		local rgNew = {}
		
		for index = 1, GetNumCompanions("MOUNT"), 1 do
			local creatureID, creatureName = GetCompanionInfo("MOUNT", index)
			local fMatchMade, iRank = self:GetMountRankAndTypesFromSavedData(iMatch, creatureID)

			if (fMatchMade) then 
				iMatch = iMatch + 1 
			else
				if (not self.db.profile.fFirstBoot) then
					self:Print(L["New mount added! "] .. creatureName)
					if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
						self.db.profile.idHotMount = creatureID
					end
				end
			end

			rgNew[index] = { creatureName, creatureID, iRank }
		end

		-- now that we've read everything in, blow away old saved ranking with new list
		self.db.profile.rgMountInfo = rgNew
	end

	-- we're now going to identify the speeds of the mounts and put them into the appropriate list for the UI
	for index = 1, GetNumCompanions("MOUNT"), 1 do
		local creatureID, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", index)

		if (creatureID == self.db.profile.idHotMount) then g_iHotMount = index end
		if (active) then g_iCurrentMount = index end

		g_rgstrHotMountNames[index] = creatureName

		-- derive type info
		local iSpeed = self:MountTypesFromName(creatureName, spellID)
		local iSpeedAlt = MS_Unknown -- because ATV need to be both runners *and* flyers, a mount can be in two different speed classes
		
		local iNew = 1
		if (iSpeed == MS_ATV) then
			-- these crazy mounts need to appear in both the flyer AND runner lists (provided the user knows how to fly)
			if (g_iRidingSkill == 300) then 
				iSpeedAlt = MS_Flyer280
			elseif (g_iRidingSkill == 225) then 
				iSpeedAlt = MS_Flyer60 
			end
				
			if (g_iRidingSkill >= 150) then 
				iSpeed = MS_Runner100
			else 
				iSpeed = MS_Runner60
			end
		elseif (iSpeed == MS_FlyerSkill) then
			if (g_iRidingSkill == 300) then 
				iSpeed = MS_Flyer280
			else
				iSpeed = MS_Flyer60 
			end
		elseif (iSpeed == MS_RunnerSkill) then
			if (g_iRidingSkill >= 150) then 
				iSpeed = MS_Runner100
			else 
				iSpeed = MS_Runner60
			end
		end
		
		if (iSpeed ~= MS_Unknown) then
			if (g_MountsByType[iSpeed] == nil) then g_MountsByType[iSpeed] = { {}, {} } end
			local iNew = 1 + #g_MountsByType[iSpeed][MOUNT_INDICES]

			g_MountsByType[iSpeed][MOUNT_INDICES][iNew] = index
			g_MountsByType[iSpeed][MOUNT_NAMES][iNew] = creatureName
		end
		if (iSpeedAlt ~= MS_Unknown) then
			if (g_MountsByType[iSpeedAlt] == nil) then g_MountsByType[iSpeedAlt] = { {}, {} } end
			local iNew = 1 + #g_MountsByType[iSpeedAlt][MOUNT_INDICES]

			g_MountsByType[iSpeedAlt][MOUNT_INDICES][iNew] = index
			g_MountsByType[iSpeedAlt][MOUNT_NAMES][iNew] = creatureName
		end
	end
	

	-- create list of mount types based on those found
	if (g_MountsByType[MS_RunnerQiraji] ~= nil) then
		g_rgstrMountTypes[MS_RunnerQiraji] = L["Qiraji Scarabs"]
		g_iMountType = MS_RunnerQiraji
	end
	if (g_MountsByType[MS_Runner0] ~= nil) then
		g_rgstrMountTypes[MS_Runner0] = L["Slow Runners"]
		g_iMountType = MS_Runner0
	end
	if (g_MountsByType[MS_Runner60] ~= nil) then
		g_rgstrMountTypes[MS_Runner60] = L["Runners"]
		g_iMountType = MS_Runner60
	end
	if (g_MountsByType[MS_Runner100] ~= nil) then
		g_rgstrMountTypes[MS_Runner100] = L["Very Fast Runners"]
		g_iMountType = MS_Runner100
	end
	if (g_MountsByType[MS_Flyer60] ~= nil) then
		g_rgstrMountTypes[MS_Flyer60] = L["Flyers"]
		g_iMountType = MS_Flyer60
	end
	if (g_MountsByType[MS_Flyer280] ~= nil) then
		g_rgstrMountTypes[MS_Flyer280] = L["Very Fast Flyers"]
		g_iMountType = MS_Flyer280
	end
	if (g_MountsByType[MS_Flyer310] ~= nil) then
		g_rgstrMountTypes[MS_Flyer310] = L["Extremely Fast Flyers"]
		g_iMountType = MS_Flyer310
	end


	if (g_iMountType == 0) then
		g_rgstrMountTypes = g_rgstrNoMounts
		self.Print("no mount types found")
	else
		g_iMountOfType = 1
	end

	g_rgstrHotMountNames[0] = L["no hot mount"]

end


-- return values - fMatchMade, iRank
function Pokedex:GetMountRankAndTypesFromSavedData(iStart, iID)
	local cLimit = #self.db.profile.rgMountInfo
	local index
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgMountInfo[index][MI_ID] == iID) then
				return true, self.db.profile.rgMountInfo[index][MI_RANK]
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgMountInfo[index][MI_ID] == iID) then
				return true, self.db.profile.rgMountInfo[index][MI_RANK]
			end
		end
	end
	
	return false, g_iDefaultRank
end


function Pokedex:MountTypesFromName(creatureName, spellID)
	if (nil == creatureName) then return MS_Unknown end
	
	-- these four erroneously claim only very fast when they are in fact extremely fast
	if (strfind(creatureName, "Ashes of Al'ar") or
		strfind(creatureName, "Black Proto-Drake") or
		strfind(creatureName, "Plagued Proto-Drake") or
		strfind(creatureName, "Violet Proto-Drake")) then
		return MS_Flyer310 

	elseif (strfind(creatureName, "Headless Horseman's Mount")) then
		return MS_ATV

	elseif (strfind(creatureName, "Black Qiraji Battle Tank")) then
		return MS_Runner100  -- I'm so punting on getting the black Qiraji into both lists, not worth the effort
	elseif (strfind(creatureName, "Qiraji")) then
		return MS_RunnerQiraji
	
	elseif (strfind(creatureName, "Riding Turtle")) then
		return MS_Runner0
		
	end
	
	local fExtremelyFast = false
	local fVeryFast = false
	local fRidingSkill = false
	local fFlyer = false
	
	PokedexScanningTooltip:ClearLines()
	PokedexScanningTooltip:SetHyperlink("spell:"..spellID) 
	
	for iLines=1,PokedexScanningTooltip:NumLines() do
		local TipTextObject = getglobal("PokedexScanningTooltipTextLeft" .. iLines)
		local strText = TipTextObject:GetText()
		-- self:Print(strText)
		
		if strfind(strText, L["extremely fast"]) then 
			fExtremelyFast = true
		elseif strfind(strText, L["very fast"]) then 
			fVeryFast = true 
		elseif strfind(strText, L["Riding skill"]) then 
			fRidingSkill = true 
		end
		
		if (strfind(strText, L["only"]) and
		    strfind(strText, L["Outland"]) and
		    strfind(strText, L["Northrend"])) then 
			fFlyer = true
		end
	end

	if fFlyer then
		if fExtremelyFast then return MS_Flyer310
		elseif fVeryFast then return MS_Flyer280
		elseif fRidingSkill then return MS_FlyerSkill
		else return MS_Flyer60 end
	else
		if fVeryFast then return MS_Runner100
		elseif fRidingSkill then return MS_RunnerSkill
		else return MS_Runner60 end
	end
	
end

--=========================================================================--
-- Mount options UI functions
--=========================================================================--

-- CtrlMountTypes
function Pokedex:MountTypeValues(info)
	if (g_iMountType == 0) then
		return g_rgstrNoMounts
	else
		return g_rgstrMountTypes
	end
end

function Pokedex:GetMountType(info) return g_iMountType; end
function Pokedex:SetMountType(info, iNew) 
	g_iMountType = iNew
	if (g_MountsByType[iNew] ~= nil) then
		g_iMountOfType = 1
	else
		self.Print("ERROR: category selected that appears to have no mounts")
		g_iMountOfType = 0
	end
end

-- CtrlMountsOfType
function Pokedex:MountsOfTypeValues(info)
	if (g_iMountOfType == 0 or g_MountsByType[g_iMountType] == nil) then
		return g_rgstrNoMounts
	else
		return g_MountsByType[g_iMountType][MOUNT_NAMES]
	end
end

function Pokedex:GetMountOfType(info) return g_iMountOfType; end
function Pokedex:SetMountOfType(info, iNew) g_iMountOfType = iNew; end

-- CtrlRankingForMount
function Pokedex:FDisablePerMountSettings(info) return (g_iMountOfType == 0 or g_iMountType == 0); end

function Pokedex:GetRankingForMount(info)
	if (g_iMountOfType > 0 and g_iMountType > 0) then
		local iMount = g_MountsByType[g_iMountType][MOUNT_INDICES][g_iMountOfType]
		return self.db.profile.rgMountInfo[iMount][MI_RANK]
	else
		return g_iDefaultRank
	end
end

function Pokedex:SetRankingForMount(info, iNew)
	local iMount = g_MountsByType[g_iMountType][MOUNT_INDICES][g_iMountOfType]
	self.db.profile.rgMountInfo[iMount][MI_RANK] = iNew
end

-- CtrlHotMount
function Pokedex:HotMountValues(info) return g_rgstrHotMountNames; end
function Pokedex:GetHotMount(info) return g_iHotMount; end
function Pokedex:SetHotMount(info, iNew)
	g_iHotMount = iNew
	if (iNew > 0) then
		self.db.profile.idHotMount = self.db.profile.rgMountInfo[iNew][MI_ID]
	else
		self.db.profile.idHotMount = 0;
	end
end

-- CtrlMountHeat
function Pokedex:GetMountHeat(info) return self.db.profile.iHotMountPref; end
function Pokedex:SetMountHeat(info, iNew) self.db.profile.iHotMountPref = iNew; end


--=========================================================================--
-- Companion functions
--=========================================================================--

function Pokedex:SummonCompanion()
	self:GetCurrentCompanion()
	
	if (g_iCurrentCompanion ~= 0) then
		local start, duration, enable = GetCompanionCooldown("CRITTER", g_iCurrentCompanion)
		if (duration ~= 0) then
			-- self:Print("in cooldown, ignore this summons attempt")
			return
		end
	end

	local INDEX = 1
	local RANK = 2
	local rgSummonable = {}	-- { INDEX, RANK }
	local iNew = 1
	local cRanks = 0
	local fHasSnowballs = GetItemCount(g_itemIDSnowball) > 0

	for iCompanion = 1, #self.db.profile.rgCompanionInfo, 1 do
		-- if its the current companion, then it is skipped
		if (iCompanion ~= g_iCurrentCompanion) then
			-- if its ranked as 0, then it gets skipped
			if (self.db.profile.rgCompanionInfo[iCompanion][CI_RANK] > 0) then
				-- if we don't have snowballs and this pet requires one, then it is skipped
				if (fHasSnowballs or not self.db.profile.rgCompanionInfo[iCompanion][CI_SNOWBALL]) then
					-- if hot pet and hotness is enabled then give it the hot chance only
					if (self.db.profile.fEnableHotness and iCompanion == g_iHotCompanion) then
						if (self.db.profile.iHotCompanionPref >= math.random(1,100)) then
							CallCompanion("CRITTER", g_iHotCompanion)
							self:AnnounceSummon(g_iHotCompanion)
							return
						end
					-- else its put into the pool of summonables
					else
						rgSummonable[iNew] = { iCompanion, self.db.profile.rgCompanionInfo[iCompanion][CI_RANK] }
						cRanks = cRanks + self.db.profile.rgCompanionInfo[iCompanion][CI_RANK]
						iNew = iNew + 1
					end
				end
			end
		end
	end

--	for iNew = 1, #rgSummonable, 1 do
--		local realindex = rgSummonable[iNew][INDEX]
--		self:Print(self.db.profile.rgCompanionInfo[realindex][CI_RANK] .. "  " .. self.db.profile.rgCompanionInfo[realindex][CI_NAME])
--	end


	if (cRanks <= 0) then 
		if (g_iCurrentCompanion ~= 0) then
			return -- only summonable pet is already summoned
		end
		
		self:Print(L["ERROR: You have no summonable companions."])
		return
	end
	
	cRanks = math.random(1,cRanks)

	for iNew = 1, #rgSummonable, 1 do
		cRanks = cRanks - rgSummonable[iNew][RANK]
		if (cRanks <= 0) then 
			-- found our slot
			CallCompanion("CRITTER", rgSummonable[iNew][INDEX])
			self:AnnounceSummon(rgSummonable[iNew][INDEX])
			return
		end  
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]) end
end


function Pokedex:SummonNextCompanion()
	self:GetCurrentCompanion()
	
	if (g_iCurrentCompanion ~= 0) then
		local start, duration, enable = GetCompanionCooldown("CRITTER", g_iCurrentCompanion)
		if (duration ~= 0) then
			-- self:Print("in cooldown, ignore this summons attempt")
			return
		end
	end

	local cRanks = 0
	local fHasSnowballs = GetItemCount(g_itemIDSnowball) > 0

	for iCompanion = g_iCurrentCompanion + 1, #self.db.profile.rgCompanionInfo, 1 do
		-- if its ranked as 0, then it gets skipped
		if (self.db.profile.rgCompanionInfo[iCompanion][CI_RANK] > 0) then
			-- if we don't have snowballs and this pet requires one, then it is skipped
			if (fHasSnowballs or not self.db.profile.rgCompanionInfo[iCompanion][CI_SNOWBALL]) then
				CallCompanion("CRITTER", iCompanion)
				self:AnnounceSummon(iCompanion)
				return
			end
		end
	end

	for iCompanion = 1, g_iCurrentCompanion, 1 do
		-- if its ranked as 0, then it gets skipped
		if (self.db.profile.rgCompanionInfo[iCompanion][CI_RANK] > 0) then
			-- if we don't have snowballs and this pet requires one, then it is skipped
			if (fHasSnowballs or not self.db.profile.rgCompanionInfo[iCompanion][CI_SNOWBALL]) then
				CallCompanion("CRITTER", iCompanion)
				self:AnnounceSummon(iCompanion)
				return
			end
		end
	end
end


function Pokedex:DismissCompanion()
	local iCurrent = self:GetCurrentCompanion()
	if (iCurrent > 0) then
		CallCompanion("CRITTER", iCurrent) 
	end 
end


function Pokedex:ToggleCompanion()
	if (self:GetCurrentCompanion() > 0) then
		self:DismissCompanion()
	else
		self:SummonCompanion()
	end
end


function Pokedex:GetCurrentCompanion()
	local creatureID, creatureName, spellID, icon, active

	if (g_iCurrentCompanion ~= nil and g_iCurrentCompanion~=0) then
		creatureID, creatureName, spellID, icon, active = GetCompanionInfo("CRITTER", g_iCurrentCompanion)
		if (active) then 
			return g_iCurrentCompanion
		end 
	end
	
	g_iCurrentCompanion = 0 -- might not be any companion active anymore
	
	for index = 1, GetNumCompanions("CRITTER"), 1 do
		creatureID, creatureName, spellID, icon, active = GetCompanionInfo("CRITTER", index)
		if active then 
			g_iCurrentCompanion = index
			break
		end
	end
	
	return g_iCurrentCompanion
end


function Pokedex:UpdateCompanionInfo()
	g_iCurrentCompanion = 0
	g_rgstrCompanionNames = {}
	g_iCompanionForRanking = 0
	g_rgstrHotCompanionNames = {}
	g_iHotCompanion = 0

	-- assumption is that order doesn't change except for insertions of new pets
	-- so we can speed up searching saved data for rankings by starting the search 
	-- at a predicted index
	local iMatch = 1
	local rgNew = {}

	for index = 1, GetNumCompanions("CRITTER"), 1 do
		local creatureID, creatureName, spellID, icon, active = GetCompanionInfo("CRITTER", index)
		local fMatchMade, iRank = self:GetCompanionRankFromSavedData(iMatch, creatureID)
		local fSnowball = self:GetCompanionAttributes(creatureName, spellID)

		if (fMatchMade) then 
			iMatch = iMatch + 1 
		elseif (not self.db.profile.fFirstBoot) then
			self:Print(L["New companion added! "] .. creatureName)
			if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
				self.db.profile.idHotCompanion = creatureID
			end
		end

		if (creatureID == self.db.profile.idHotCompanion) then g_iHotCompanion = index end
		if (active) then g_iCurrentCompanion = index end

		g_rgstrCompanionNames[index] = creatureName
		g_rgstrHotCompanionNames[index] = creatureName
		rgNew[index] = { creatureName, creatureID, iRank, fSnowball }
	end

	if (#g_rgstrCompanionNames < 1) then
		g_iCompanionForRanking = 0
	elseif (g_iCurrentCompanion > 0) then
		g_iCompanionForRanking = g_iCurrentCompanion
	else
		g_iCompanionForRanking = 1
	end

	g_rgstrHotCompanionNames[0] = L["no hot companion"]

	-- now that we've read everything in, blow away old saved ranking with new list
	self.db.profile.rgCompanionInfo = rgNew
end


function Pokedex:GetCompanionRankFromSavedData(iStart, iID)
	local cLimit = #self.db.profile.rgCompanionInfo
	local index
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgCompanionInfo[index][CI_ID] == iID) then
				return true, self.db.profile.rgCompanionInfo[index][CI_RANK]
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgCompanionInfo[index][CI_ID] == iID) then
				return true, self.db.profile.rgCompanionInfo[index][CI_RANK]
			end
		end
	end
	
	return false, g_iDefaultRank
end


function Pokedex:GetCompanionAttributes(creatureName, spellID)
	local fNeedsSnowball = false
	
	PokedexScanningTooltip:ClearLines()
	PokedexScanningTooltip:SetHyperlink("spell:"..spellID) 
	
	for iLines=1,PokedexScanningTooltip:NumLines() do
		local TipTextObject = getglobal("PokedexScanningTooltipTextLeft" .. iLines)
		local strText = TipTextObject:GetText()
		-- self:Print(strText)
		
		if strfind(strText, L["Requires a Snowball"]) then 
			fNeedsSnowball = true
		end
	end
	
	return fNeedsSnowball
end


function Pokedex:CheckBags()
	-- bags are 0-4 inclusive
--	local iBag = 0
--	local cBagSlots = 0
--	local itemLink
	
--	cBagSlots = GetContainerNumSlots(iBag)
--	for iSlot=1, cBagSlots
--		itemLink = GetContainerItemLink(iBag,iSlot)
		
--		17202

	local snowballs = GetItemCount(17202)
	self:Print("number of snowballs is " .. snowballs)
end		
		


--=========================================================================--
-- Companion options UI functions
--=========================================================================--

-- CtrlCompanionForRanking
function Pokedex:CompanionForRankingValues(info)
	if (g_iCompanionForRanking == 0) then
		return g_rgstrNoCompanions
	else
		return g_rgstrCompanionNames
	end
end

function Pokedex:GetCompanionForRanking(info) return g_iCompanionForRanking; end
function Pokedex:SetCompanionForRanking(info, iNew) g_iCompanionForRanking = iNew; end

-- CtrlRankingForCompanion
function Pokedex:FDisableRankingForCompanion(info) return not (#self.db.profile.rgCompanionInfo > 0); end

function Pokedex:GetRankingForCompanion(info)
	if (#self.db.profile.rgCompanionInfo > 0) then
		return self.db.profile.rgCompanionInfo[g_iCompanionForRanking][CI_RANK]
	else
		return g_iDefaultRank
	end
end

function Pokedex:SetRankingForCompanion(info, iNew)
	self.db.profile.rgCompanionInfo[g_iCompanionForRanking][CI_RANK] = iNew
end

-- CtrlHotCompanion
function Pokedex:HotCompanionValues(info) return g_rgstrHotCompanionNames; end
function Pokedex:GetHotCompanion(info) return g_iHotCompanion; end
function Pokedex:SetHotCompanion(info, iNew)
	g_iHotCompanion = iNew
	if (iNew > 0) then
		self.db.profile.idHotCompanion = self.db.profile.rgCompanionInfo[iNew][CI_ID]
	else
		self.db.profile.idHotCompanion = 0;
	end
end

-- CtrlCompanionHeat
function Pokedex:GetCompanionHeat(info) return self.db.profile.iHotCompanionPref; end
function Pokedex:SetCompanionHeat(info, iNew) self.db.profile.iHotCompanionPref = iNew; end


--=========================================================================--
-- Title functions
--=========================================================================--

function Pokedex:ChangeTitle()
	local iCurrentTitle = GetCurrentTitle()
	local iCurrent = 0
	
	local cRanks = 0
	for iRanks = 1, #self.db.profile.rgTitleInfo, 1 do
		cRanks = cRanks + self.db.profile.rgTitleInfo[iRanks][TI_RANK]
		if self.db.profile.rgTitleInfo[iRanks][TI_ID] == iCurrentTitle then
			iCurrent = iRanks
		end
	end

	if (self.db.profile.fEnableHotness and g_iHotTitle ~= 0 and g_iHotTitle ~= iCurrent) then
		-- do hot check and possibly change
		if (self.db.profile.iHotTitlePref >= math.random(1,100)) then
			SetCurrentTitle(self.db.profile.rgTitleInfo[g_iHotTitle][TI_ID])
			return
		end
		
		-- if you're here, they have a hot title, but its getting set
		cRanks = cRanks - self.db.profile.rgTitleInfo[g_iHotTitle][TI_RANK]
	end
	
	if (iCurrent ~= 0) then
		cRanks = cRanks - self.db.profile.rgTitleInfo[iCurrent][TI_RANK]
	end

	if (cRanks <= 0) then 
		if (iCurrent ~= 0) then
			return -- only title is already displayed
		end
		
		-- self:Print(L["ERROR: You don't have any titles."])
		return
	end
	
	cRanks = math.random(1,cRanks)

	for iNew = 1, #self.db.profile.rgTitleInfo, 1 do
		if (iNew ~= iCurrent and (iNew ~= g_iHotTitle or not self.db.profile.fEnableHotness)) then
			cRanks = cRanks - self.db.profile.rgTitleInfo[iNew][TI_RANK]
			if (cRanks <= 0) then 
				-- found our slot
				SetCurrentTitle(self.db.profile.rgTitleInfo[iNew][TI_ID])
				return
			end  
		end
	end
	
	if (cRanks > 0) then self:Print(L["ERROR: selection error"]) end
end


function Pokedex:UpdateTitleInfo()
	g_rgstrTitleNames = {}
	g_iTitleForRanking = 0
	g_rgstrHotTitleNames = {}
	g_iHotTitle = 0
	
	-- assumption is that order doesn't change except for insertions of new titles
	-- so we can speed up searching saved data for rankings by starting the search 
	-- at a predicted index
	local cTitlesKnown = 0
	local iMatch = 1
	local rgNew = {}
	
	for iID = 1, GetNumTitles(), 1 do
		if 1 == IsTitleKnown(iID) then
			cTitlesKnown = cTitlesKnown + 1
			local titleName = GetTitleName(iID)
			local fMatchMade, iRank = self:GetTitleRankFromSavedData(iMatch, iID)

			if (fMatchMade) then 
				iMatch = iMatch + 1 
			elseif (not self.db.profile.fFirstBoot) then
				self:Print(L["New title added! "] .. titleName)
				if (self.db.profile.fEnableHotness and self.db.profile.fNewHotness) then
					self.db.profile.idHotTitle = iID
				end
			end
			
			if (self.db.profile.idHotTitle == iID) then g_iHotTitle = cTitlesKnown end
						
			g_rgstrTitleNames[cTitlesKnown] = titleName
			g_rgstrHotTitleNames[cTitlesKnown] = titleName
			rgNew[cTitlesKnown] = { titleName, iID, iRank }
		end
	end

	if (#g_rgstrTitleNames < 1) then
		g_iTitleForRanking = 0
	else
		g_iTitleForRanking = 1
	end

	g_rgstrHotTitleNames[0] = L["no hot title"]
	
	-- now that we've read everything in, blow away old saved ranking with new list
	self.db.profile.rgTitleInfo = rgNew
end


function Pokedex:GetTitleRankFromSavedData(iStart, iID)
	local cLimit = #self.db.profile.rgTitleInfo
	local index
	
	if (iStart <= cLimit) then
		for index = iStart, cLimit, 1 do
			if (self.db.profile.rgTitleInfo[index][TI_ID] == iID) then
				return true, self.db.profile.rgTitleInfo[index][TI_RANK]
			end
		end
		
		for index = 1, iStart - 1, 1 do
			if (self.db.profile.rgTitleInfo[index][TI_ID] == iID) then
				return true, self.db.profile.rgTitleInfo[index][TI_RANK]
			end
		end
	end
	
	return false, g_iDefaultRank
end


--=========================================================================--
-- Title options UI functions
--=========================================================================--

-- CtrlChangeTitleOnLoad
function Pokedex:FGetChangeTitleOnLoad(info) return self.db.profile.fChangeTitleOnLoad; end
function Pokedex:SetChangeTitleOnLoad(info, fNew) self.db.profile.fChangeTitleOnLoad = fNew; end

-- CtrlTitleForRanking
function Pokedex:TitleForRankingValues(info)
	if (g_iTitleForRanking == 0) then
		return g_rgstrNoTitles
	else
		return g_rgstrTitleNames
	end
end

function Pokedex:GetTitleForRanking(info) return g_iTitleForRanking; end
function Pokedex:SetTitleForRanking(info, iNew) g_iTitleForRanking = iNew; end

-- CtrlRankingForTitle
function Pokedex:FDisableRankingForTitle(info) return not (#self.db.profile.rgTitleInfo > 0); end

function Pokedex:GetRankingForTitle(info)
	if (#self.db.profile.rgTitleInfo > 0) then
		return self.db.profile.rgTitleInfo[g_iTitleForRanking][TI_RANK]
	else
		return g_iDefaultRank
	end
end

function Pokedex:SetRankingForTitle(info, iNew)
	self.db.profile.rgTitleInfo[g_iTitleForRanking][TI_RANK] = iNew
end

-- CtrlHotTitle
function Pokedex:HotTitleValues(info) return g_rgstrHotTitleNames; end
function Pokedex:GetHotTitle(info) return g_iHotTitle; end
function Pokedex:SetHotTitle(info, iNew)
	g_iHotTitle = iNew
	if (iNew > 0) then
		self.db.profile.idHotTitle = self.db.profile.rgTitleInfo[iNew][TI_ID]
	else
		self.db.profile.idHotTitle = 0;
	end
end

-- CtrlTitleHeat
function Pokedex:GetTitleHeat(info) return self.db.profile.iHotTitlePref; end
function Pokedex:SetTitleHeat(info, iNew) self.db.profile.iHotTitlePref = iNew; end
